/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2008 OW2 Consortium, http://www.ow2.org/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.component.framework.constants;

import java.net.URI;

/**
 * Soap envelope constant
 * @author Mathieu CARROLLE
 * @author Christophe DENEUX (Capgemini Sud)
 *
 */
public enum SoapEnvelope {
    SOAP11_NS("http://schemas.xmlsoap.org/soap/envelope/"),
    SOAP12_NS("http://www.w3.org/2003/05/soap-envelope");
    
    /**
     * 
     * @param pattern
     * @return
     */
    public static SoapEnvelope valueOf(final URI pattern) {
        SoapEnvelope result = null;
        if (pattern != null) {
            for (SoapEnvelope soap : SoapEnvelope.values()) {
                if (soap.nameSpace.equals(pattern.toString())) {
                    result = soap;
                }
            }
        }
        return result;
    }
    
    private final String nameSpace;
    private final URI soapURI;
    
    private SoapEnvelope(final String nameSpace) {
        this.nameSpace = nameSpace;
        this.soapURI = URI.create(nameSpace);
    }
    
    /**
     * 
     * @return
     */
    public URI value() {
        return this.soapURI;
    }

    /**
     * Please use this equals method instead of using :<code>
     * value().equals(mep)
     * </code>
     * which is almost 10 times slower...
     * 
     * @param mep
     * @return
     */
    public boolean equals(final URI mep) {
        return this.nameSpace.equals(mep.toString());
    }
}