/**
 * PETALS - PETALS Services Platform. Copyright (c) 2005 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * ------------------------------------------------------------------------- $Id
 * $ -------------------------------------------------------------------------
 */

package org.ow2.petals.component.framework.message;

import java.net.URI;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import javax.activation.DataHandler;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.Fault;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.security.auth.Subject;
import javax.xml.namespace.QName;
import javax.xml.transform.Source;

import org.ow2.petals.component.framework.Constants.MEPConstants;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.util.NormalizedMessageUtil;
import org.ow2.petals.component.framework.util.SourceHelper;
import org.w3c.dom.Document;

/**
 * This is a Message Exchange Wrapper. The "in" and "out" normalized message are
 * cached into this class, authorizing the reset of these message. Can be use by
 * the Component Framework interceptors.
 * 
 * @author Adrien LOUIS - EBMWebSourcing
 * @author ofabre - EBM WebSourcing
 * @since 3.0
 * 
 */
public class ExchangeImpl implements Exchange {

    private MessageExchange exchange;

    private NormalizedMessage inNormalizedExchange;

    private NormalizedMessage outNormalizedExchange;

    private boolean checkRole;

    /**
     * 
     * Creates a new instance of {@link ExchangeImpl}
     * 
     * @param exchange
     */
    public ExchangeImpl(MessageExchange exchange) {
        super();
        this.exchange = exchange;
        this.inNormalizedExchange = this.exchange.getMessage(IN_MESSAGE_NAME);
        this.outNormalizedExchange = this.exchange.getMessage(OUT_MESSAGE_NAME);
        this.checkRole = true;
    }

    /**
     * Creates a message.
     * <p>
     * This method is not exposed by the {@code Exchange} interface.
     * 
     * @return
     * @throws MessagingException
     */
    public NormalizedMessage createMessage() throws MessagingException {
        return this.exchange.createMessage();
    }

    /**
     * Return the wrapped Message Exchange. Set the "in" or the "out" normalized
     * messages if they are not already set.
     * <p>
     * This method is not exposed by the {@code Exchange} interface.
     * 
     * @return the wrapped Message Exchange
     */
    public MessageExchange getMessageExchange() {
        try {
            if (this.inNormalizedExchange != null
                    && this.exchange.getMessage(IN_MESSAGE_NAME) == null) {
                this.exchange.setMessage(this.inNormalizedExchange, IN_MESSAGE_NAME);
            }

            if (this.outNormalizedExchange != null
                    && this.exchange.getMessage(OUT_MESSAGE_NAME) == null) {
                this.exchange.setMessage(this.outNormalizedExchange, OUT_MESSAGE_NAME);
            }
        } catch (MessagingException e) {
            // do nothing
        }

        return this.exchange;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#createFault()
     */
    public Fault createFault() throws MessagingException {
        return this.exchange.createFault();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#getEndpoint()
     */
    public ServiceEndpoint getEndpoint() {
        return this.exchange.getEndpoint();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.api.message.Exchange#getError()
     */
    public Exception getError() {
        return this.exchange.getError();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#getExchangeId()
     */
    public String getExchangeId() {
        return this.exchange.getExchangeId();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.api.message.Exchange#getFault()
     */
    public Fault getFault() {
        return this.exchange.getFault();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#getInterfaceName
     * ()
     */
    public QName getInterfaceName() {
        return this.exchange.getInterfaceName();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#getOperation()
     */
    public QName getOperation() {
        return this.exchange.getOperation();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.api.message.Exchange#getPattern()
     */
    public URI getPattern() {
        return this.exchange.getPattern();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#getProperty(java
     * .lang.String)
     */
    public Object getProperty(final String name) {
        return this.exchange.getProperty(name);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#getPropertyNames
     * ()
     */
    public Set<?> getPropertyNames() {
        return this.exchange.getPropertyNames();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.api.message.Exchange#getRole()
     */
    public MessageExchange.Role getRole() {
        return this.exchange.getRole();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.api.message.Exchange#getService()
     */
    public QName getService() {
        return this.exchange.getService();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.api.message.Exchange#getStatus()
     */
    public ExchangeStatus getStatus() {
        return this.exchange.getStatus();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#isTransacted()
     */
    public boolean isTransacted() {
        return this.exchange.isTransacted();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#setEndpoint(javax
     * .jbi.servicedesc.ServiceEndpoint)
     */
    public void setEndpoint(final ServiceEndpoint endpoint) {
        this.exchange.setEndpoint(endpoint);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#setError(java
     * .lang.Exception)
     */
    public void setError(final Exception error) {
        String packageName = error.getClass().getPackage().getName();
        if (packageName.startsWith("java") || packageName.startsWith("org.w3c")
                || packageName.startsWith("org.xml")) {
            this.exchange.setError(error);
        } else {
            throw new IllegalArgumentException("Exception of type '" + packageName
                    + "' are not supported by the JBI container");
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#setFault(javax
     * .jbi.messaging.Fault)
     */
    public void setFault(Fault fault) throws MessagingException {
        this.exchange.setFault(fault);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#setFault(java
     * .lang.Exception)
     */
    public void setFault(Exception e) throws MessagingException {
        final Fault fault = this.exchange.createFault();
        final Document faultDocument = SourceHelper.createSoapFaultDocument(e,
                Exchange.DEFAULT_FAULT_QNAME);

        if (SourceHelper.isDOM3(faultDocument)) {
            fault.setContent(SourceHelper.createSource(faultDocument));
        } else {
            Source inSource;
            try {
                inSource = SourceHelper.convertDOM2ToSource(faultDocument);
            } catch (PEtALSCDKException e1) {
                throw new MessagingException(e1);
            }
            fault.setContent(inSource);
        }

        this.exchange.setFault(fault);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#setInterfaceName
     * (javax.xml.namespace.QName)
     */
    public void setInterfaceName(QName interfaceName) {
        this.exchange.setInterfaceName(interfaceName);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#setOperation(
     * javax.xml.namespace.QName)
     */
    public void setOperation(QName name) {
        this.exchange.setOperation(name);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#setProperty(java
     * .lang.String, java.lang.Object)
     */
    public void setProperty(String name, Object obj) {
        this.exchange.setProperty(name, obj);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#setService(javax
     * .xml.namespace.QName)
     */
    public void setService(QName service) {
        this.exchange.setService(service);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#setStatus(javax
     * .jbi.messaging.ExchangeStatus)
     */
    public void setStatus(ExchangeStatus status) throws MessagingException {
        this.exchange.setStatus(status);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#setOutMessageContent
     * (java.lang.String)
     */
    public void setOutMessageContent(String outContent) throws MessagingException {
        try {
            NormalizedMessage outMess = this.getOutMessage();
            if (outMess == null) {
                outMess = this.exchange.createMessage();
            }
            outMess.setContent(SourceHelper.createSourceFromString(outContent));
            this.setOutMessage(outMess);
        } catch (PEtALSCDKException e) {
            throw new MessagingException("Error creating response message");
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#setOutMessageContent
     * (org.w3c.dom.Document)
     */
    public void setOutMessageContent(Document outContent) throws MessagingException {
        NormalizedMessage outMess = this.getOutMessage();
        if (outMess == null) {
            outMess = this.exchange.createMessage();
        }

        if (SourceHelper.isDOM3(outContent)) {
            outMess.setContent(SourceHelper.createSource(outContent));
        } else {
            Source inSource;
            try {
                inSource = SourceHelper.convertDOM2ToSource(outContent);
            } catch (PEtALSCDKException e) {
                throw new MessagingException(e);
            }
            outMess.setContent(inSource);
        }

        this.setOutMessage(outMess);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#setOutMessageContent
     * (javax.xml.transform.Source)
     */
    public void setOutMessageContent(Source outContent) throws MessagingException {
        NormalizedMessage outMess = this.getOutMessage();
        if (outMess == null) {
            outMess = this.exchange.createMessage();
        }
        outMess.setContent(outContent);
        this.setOutMessage(outMess);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#setInMessageContent
     * (java.lang.String)
     */
    public void setInMessageContent(String inContent) throws MessagingException {
        try {
            NormalizedMessage inMess = this.getInMessage();
            if (inMess == null) {
                inMess = this.exchange.createMessage();
            }
            inMess.setContent(SourceHelper.createSourceFromString(inContent));
            this.setInMessage(inMess);
        } catch (PEtALSCDKException e) {
            throw new MessagingException("Error creating request message");
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#setInMessageContent
     * (org.w3c.dom.Document)
     */
    public void setInMessageContent(Document inContent) throws MessagingException {
        NormalizedMessage inMess = this.getInMessage();
        if (inMess == null) {
            inMess = this.exchange.createMessage();
        }

        if (SourceHelper.isDOM3(inContent)) {
            inMess.setContent(SourceHelper.createSource(inContent));
        } else {
            Source inSource;
            try {
                inSource = SourceHelper.convertDOM2ToSource(inContent);
            } catch (PEtALSCDKException e) {
                throw new MessagingException(e);
            }
            inMess.setContent(inSource);
        }

        this.setInMessage(inMess);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#setInMessageContent
     * (javax.xml.transform.Source)
     */
    public void setInMessageContent(Source inContent) throws MessagingException {
        NormalizedMessage inMess = this.getInMessage();
        if (inMess == null) {
            inMess = this.exchange.createMessage();
        }
        inMess.setContent(inContent);
        this.setInMessage(inMess);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.message.Exchange#
     * setOutMessageAttachments(java.util.Map)
     */
    public void setOutMessageAttachments(Map<String, DataHandler> attachments)
            throws MessagingException {
        NormalizedMessage outMess = this.getOutMessage();
        if (outMess == null) {
            outMess = this.exchange.createMessage();
        }
        for (String attachementId : attachments.keySet()) {
            outMess.addAttachment(attachementId, attachments.get(attachementId));
        }
        this.setOutMessage(outMess);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.message.Exchange#
     * setInMessageAttachments(java.util.Map)
     */
    public void setInMessageAttachments(Map<String, DataHandler> attachments)
            throws MessagingException {
        NormalizedMessage inMess = this.getInMessage();
        if (inMess == null) {
            inMess = this.exchange.createMessage();
        }
        for (String attachementId : attachments.keySet()) {
            inMess.addAttachment(attachementId, attachments.get(attachementId));
        }
        this.setInMessage(inMess);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.message.Exchange#
     * getInMessageContentAsString()
     */
    @Deprecated
    public String getInMessageContentAsString() throws MessagingException {
        NormalizedMessage inMessage = this.getInMessage();
        if (inMessage == null) {
            throw new MessagingException("In message must not be null");
        }
        Source content = inMessage.getContent();
        String stringContent = null;
        try {
            stringContent = SourceHelper.convertToString(content);
        } catch (PEtALSCDKException e) {
            throw new MessagingException("Error creating string from source", e);
        }
        return stringContent;
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.message.Exchange#
     * getInMessageContentAsDocument()
     */
    public Document getInMessageContentAsDocument() throws MessagingException {
        return this.getInMessageContentAsDocument(false);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.message.Exchange#
     * getInMessageContentAsDocument(boolean)
     */
    public Document getInMessageContentAsDocument(boolean forkStreamSource)
            throws MessagingException {
        final NormalizedMessage message = this.getInMessage();
        return NormalizedMessageUtil.createNewDocumentFromMessageContent(message, forkStreamSource);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.message.Exchange#
     * getInMessageContentAsSource()
     */
    public Source getInMessageContentAsSource() throws MessagingException {
        NormalizedMessage inMessage = this.getInMessage();
        if (inMessage == null) {
            throw new MessagingException("In message must not be null");
        }
        return inMessage.getContent();
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.message.Exchange#
     * getOutMessageContentAsString()
     */
    @Deprecated
    public String getOutMessageContentAsString() throws MessagingException {
        NormalizedMessage outMessage = this.getOutMessage();
        if (outMessage == null) {
            throw new MessagingException("Out message must not be null");
        }
        Source content = outMessage.getContent();
        String stringContent = null;
        try {
            stringContent = SourceHelper.convertToString(content);
        } catch (PEtALSCDKException e) {
            throw new MessagingException("Error creating string from source", e);
        }
        return stringContent;
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.message.Exchange#
     * getOutMessageContentAsDocument()
     */
    public Document getOutMessageContentAsDocument() throws MessagingException {
        return this.getOutMessageContentAsDocument(false);
    }

    public Document getOutMessageContentAsDocument(boolean forkStreamSource)
            throws MessagingException {
        final NormalizedMessage outMessage = this.getOutMessage();
        return NormalizedMessageUtil.createNewDocumentFromMessageContent(outMessage,
                forkStreamSource);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.message.Exchange#
     * getOutMessageContentAsSource()
     */
    public Source getOutMessageContentAsSource() throws MessagingException {
        NormalizedMessage outMessage = this.getOutMessage();
        if (outMessage == null) {
            throw new MessagingException("Out message must not be null");
        }
        return outMessage.getContent();
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.message.Exchange#
     * getInMessageAttachments()
     */
    public Set<DataHandler> getInMessageAttachments() throws MessagingException {
        Set<DataHandler> attachments = new HashSet<DataHandler>();

        NormalizedMessage inMessage = this.getInMessage();
        if (inMessage == null) {
            throw new MessagingException("In message must not be null");
        }

        Set<?> names = inMessage.getAttachmentNames();
        for (Object attachId : names) {
            DataHandler dh = inMessage.getAttachment((String) attachId);
            if (dh != null)
                attachments.add(dh);
        }
        return attachments;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#getExchangePattern
     * ()
     */
    public URI getExchangePattern() throws MessagingException {
        URI pattern = this.exchange.getPattern();
        if (pattern == null) {
            throw new MessagingException(
                    "You must specify a message exchange pattern in the message exchange");
        }
        return pattern;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#getOperationName
     * ()
     */
    public String getOperationName() {
        QName operation = this.exchange.getOperation();
        if (operation == null) {
            return null;
        }
        return operation.getLocalPart();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#getEndpointName()
     */
    public String getEndpointName() throws MessagingException {
        ServiceEndpoint ep = this.exchange.getEndpoint();
        if (ep == null) {
            throw new MessagingException("You must specify an endpoint in the message exchange");
        }
        return ep.getEndpointName();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#getInMessage()
     */
    public NormalizedMessage getInMessage() {
        if (this.inNormalizedExchange == null) {
            if (this.exchange.getMessage(IN_MESSAGE_NAME) != null) {
                return this.exchange.getMessage(IN_MESSAGE_NAME);
            }
            try {
                this.inNormalizedExchange = this.exchange.createMessage();
            } catch (MessagingException e) {
                return null;
            }
        }
        return this.inNormalizedExchange;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#isInMessage()
     */
    public boolean isInMessage() {
        return (this.exchange.getMessage(IN_MESSAGE_NAME) != null && this.exchange.getMessage(
                IN_MESSAGE_NAME).getContent() != null)
                || (this.inNormalizedExchange != null && this.inNormalizedExchange.getContent() != null);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#setInMessage(
     * javax.jbi.messaging.NormalizedMessage)
     */
    public void setInMessage(final NormalizedMessage msg) throws MessagingException {
        this.checkPatternMatching(IN_MESSAGE_NAME);
        this.checkRoleMatching(IN_MESSAGE_NAME);
        this.inNormalizedExchange = msg;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#getOutMessage()
     */
    public NormalizedMessage getOutMessage() {
        if (this.outNormalizedExchange == null) {
            if (this.exchange.getMessage(OUT_MESSAGE_NAME) != null) {
                return this.exchange.getMessage(OUT_MESSAGE_NAME);
            } else {
                try {
                    this.outNormalizedExchange = this.exchange.createMessage();
                } catch (MessagingException e) {
                    return null;
                }
            }
        }
        return this.outNormalizedExchange;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#isOutMessage()
     */
    public boolean isOutMessage() {
        return (this.exchange.getMessage(OUT_MESSAGE_NAME) != null && this.exchange.getMessage(
                OUT_MESSAGE_NAME).getContent() != null)
                || (this.outNormalizedExchange != null && this.outNormalizedExchange.getContent() != null);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#setOutMessage
     * (javax.jbi.messaging.NormalizedMessage)
     */
    public void setOutMessage(final NormalizedMessage msg) throws MessagingException {
        this.checkPatternMatching(OUT_MESSAGE_NAME);
        this.checkRoleMatching(OUT_MESSAGE_NAME);
        this.outNormalizedExchange = msg;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#setDoneStatus()
     */
    public void setDoneStatus() throws MessagingException {
        this.setStatus(ExchangeStatus.DONE);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#isDoneStatus()
     */
    public boolean isDoneStatus() {
        return ExchangeStatus.DONE.equals(this.getStatus());
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#setErrorStatus()
     */
    public void setErrorStatus() throws MessagingException {
        this.setStatus(ExchangeStatus.ERROR);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#isErrorStatus()
     */
    public boolean isErrorStatus() {
        return ExchangeStatus.ERROR.equals(this.getStatus());
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#setActiveStatus()
     */
    public void setActiveStatus() throws MessagingException {
        this.setStatus(ExchangeStatus.ACTIVE);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#isActiveStatus()
     */
    public boolean isActiveStatus() {
        return ExchangeStatus.ACTIVE.equals(this.getStatus());
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#isProviderRole()
     */
    public boolean isProviderRole() {
        return MessageExchange.Role.PROVIDER.equals(this.getRole());
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#isConsumerRole()
     */
    public boolean isConsumerRole() {
        return MessageExchange.Role.CONSUMER.equals(this.getRole());
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#isInOnlyPattern()
     */
    public boolean isInOnlyPattern() {
        return MEPConstants.IN_ONLY_PATTERN.equals(this.exchange.getPattern());
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#isRobustInOnlyPattern
     * ()
     */
    public boolean isRobustInOnlyPattern() {
        return MEPConstants.ROBUST_IN_ONLY_PATTERN.equals(this.exchange.getPattern());
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#isInOutPattern()
     */
    public boolean isInOutPattern() {
        return MEPConstants.IN_OUT_PATTERN.equals(this.exchange.getPattern());
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.message.Exchange#
     * isInOptionalOutPattern()
     */
    public boolean isInOptionalOutPattern() {
        return MEPConstants.IN_OPTIONAL_OUT_PATTERN.equals(this.exchange.getPattern());
    }

    public void setCheckRole(boolean checkRole) {
        this.checkRole = checkRole;
    }

    /**
     * Check that the exchange MEP allows the specified reference. This only
     * works for the JBI predefined MEPs. It check that reference is IN or OUT.
     * Other reference or MEP will cause an exception
     * 
     * @param name
     * @throws MessagingException
     *             the MEP does not allow the reference
     */
    protected void checkPatternMatching(String name) throws MessagingException {
        if (name == null) {
            throw new MessagingException("The reference name is not defined.");
        }

        if (OUT_MESSAGE_NAME.equalsIgnoreCase(name)
                && (MEPConstants.IN_ONLY_PATTERN.equals(this.exchange.getPattern()) || MEPConstants.ROBUST_IN_ONLY_PATTERN
                        .equals(this.exchange.getPattern()))) {
            throw new MessagingException("the MessageExchange state does not allow this operation.");
        }
    }

    /**
     * Check that the Role allows the specified reference. This only works for
     * the JBI predefined MEPs. It check that reference is IN or OUT and the
     * Role is CONSUMER or PROVIDER. Other reference will cause an exception
     * 
     * @param name
     * @throws MessagingException
     *             the Role does not allow the reference
     */
    protected void checkRoleMatching(String name) throws MessagingException {
        if (name == null) {
            throw new MessagingException("The reference name is not defined.");
        }

        // The check on the role can be unactivated for CDK intercepting purpose
        if (!this.checkRole) {
            return;
        }

        if (MessageExchange.Role.CONSUMER.equals(this.exchange.getRole())) {
            if (IN_MESSAGE_NAME.equalsIgnoreCase(name)) {
                return;
            }
        } else if (MessageExchange.Role.PROVIDER.equals(this.exchange.getRole())
                && OUT_MESSAGE_NAME.equalsIgnoreCase(name)) {
            return;
        }
        throw new MessagingException("The Role does not allow this operation.");
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#getInMessageProperty
     * (java.lang.String)
     */
    public Object getInMessageProperty(String name) throws MessagingException {
        NormalizedMessage nm = this.getInMessage();
        if (nm == null) {
            throw new MessagingException("In message must not be null");
        }
        return nm.getProperty(name);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.message.Exchange#
     * getInMessagePropertyNames()
     */
    public Set<?> getInMessagePropertyNames() throws MessagingException {
        NormalizedMessage nm = this.getInMessage();
        if (nm == null) {
            throw new MessagingException("In message must not be null");
        }
        return nm.getPropertyNames();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#getOutMessageProperty
     * (java.lang.String)
     */
    public Object getOutMessageProperty(String name) throws MessagingException {
        NormalizedMessage nm = this.getOutMessage();
        if (nm == null) {
            throw new MessagingException("Out message must not be null");
        }
        return nm.getProperty(name);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.message.Exchange#
     * getOutMessagePropertyNames()
     */
    public Set<?> getOutMessagePropertyNames() throws MessagingException {
        NormalizedMessage nm = this.getOutMessage();
        if (nm == null) {
            throw new MessagingException("Out message must not be null");
        }
        return nm.getPropertyNames();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#setInMessageProperty
     * (java.lang.String, java.lang.Object)
     */
    public void setInMessageProperty(String name, Object value) throws MessagingException {
        NormalizedMessage inMess = this.getInMessage();
        if (inMess == null) {
            inMess = this.exchange.createMessage();
        }
        inMess.setProperty(name, value);
        this.setInMessage(inMess);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.message.Exchange#
     * setInMessageProperties(java.util.Map)
     */
    public void setInMessageProperties(Map<String, Object> properties) throws MessagingException {
        NormalizedMessage inMess = this.getInMessage();
        if (inMess == null) {
            inMess = this.exchange.createMessage();
        }
        Iterator<String> iter = properties.keySet().iterator();
        while (iter.hasNext()) {
            String key = iter.next();
            inMess.setProperty(key, properties.get(key));
        }
        this.setInMessage(inMess);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.message.Exchange#
     * setOutMessageProperties(java.util.Map)
     */
    public void setOutMessageProperties(Map<String, Object> properties) throws MessagingException {
        NormalizedMessage outMess = this.getOutMessage();
        if (outMess == null) {
            outMess = this.exchange.createMessage();
        }
        Iterator<String> iter = properties.keySet().iterator();
        while (iter.hasNext()) {
            String key = iter.next();
            outMess.setProperty(key, properties.get(key));
        }
        this.setOutMessage(outMess);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#setOutMessageProperty
     * (java.lang.String, java.lang.Object)
     */
    public void setOutMessageProperty(String name, Object value) throws MessagingException {
        NormalizedMessage outMess = this.getOutMessage();
        if (outMess == null) {
            outMess = this.exchange.createMessage();
        }
        outMess.setProperty(name, value);
        this.setOutMessage(outMess);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.message.Exchange#
     * setOutMessageAttachment(java.lang.String, javax.activation.DataHandler)
     */
    public void addOutMessageAttachment(String id, DataHandler content) throws MessagingException {
        NormalizedMessage outMess = this.getOutMessage();
        if (outMess == null) {
            outMess = this.exchange.createMessage();
        }
        outMess.addAttachment(id, content);
        this.setOutMessage(outMess);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.message.Exchange#
     * setInMessageAttachment(java.lang.String, javax.activation.DataHandler)
     */
    public void addInMessageAttachment(String id, DataHandler content) throws MessagingException {
        NormalizedMessage inMess = this.getInMessage();
        if (inMess == null) {
            inMess = this.exchange.createMessage();
        }
        inMess.addAttachment(id, content);
        this.setInMessage(inMess);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#setInSubject(
     * javax.security.auth.Subject)
     */
    public void setInSubject(Subject subject) {
        this.getInMessage().setSecuritySubject(subject);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#setInSubject(
     * javax.security.auth.Subject)
     */
    public void setOutSubject(Subject subject) {
        this.getOutMessage().setSecuritySubject(subject);
    }
}
