/**
 * PETALS - PETALS Services Platform. Copyright (c) 2005 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.component.framework.su;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.net.MalformedURLException;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.management.DeploymentException;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;
import javax.xml.transform.stream.StreamSource;

import org.ow2.easywsdl.extensions.wsdl4complexwsdl.WSDL4ComplexWsdlFactory;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.Description;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.WSDL4ComplexWsdlException;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.WSDL4ComplexWsdlReader;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.WSDL4ComplexWsdlWriter;
import org.ow2.easywsdl.wsdl.api.Endpoint;
import org.ow2.easywsdl.wsdl.api.Service;
import org.ow2.easywsdl.wsdl.api.WSDLException;
import org.ow2.petals.component.framework.AbstractComponent;
import org.ow2.petals.component.framework.api.configuration.ConfigurationExtensions;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.util.PropertiesUtil;
import org.ow2.petals.component.framework.interceptor.MessageInterceptorManager;
import org.ow2.petals.component.framework.jbidescriptor.CDKJBIDescriptorException;
import org.ow2.petals.component.framework.jbidescriptor.JBIDescriptorBuilder;
import org.ow2.petals.component.framework.jbidescriptor.generated.Consumes;
import org.ow2.petals.component.framework.jbidescriptor.generated.Jbi;
import org.ow2.petals.component.framework.jbidescriptor.generated.Provides;
import org.ow2.petals.component.framework.jbidescriptor.generated.SUInterceptor;
import org.ow2.petals.component.framework.jbidescriptor.generated.SUInterceptors;
import org.ow2.petals.component.framework.jbidescriptor.generated.Settableboolean;
import org.ow2.petals.component.framework.util.IDGenerator;
import org.ow2.petals.component.framework.util.ManagementMessageUtil;
import org.ow2.petals.component.framework.util.StringHelper;
import org.ow2.petals.component.framework.util.WSDLHelper;
import org.w3c.dom.Document;

import static org.ow2.petals.component.framework.Constants.Component.AUTOGENERATED_ENDPOINT_NAME;
import static org.ow2.petals.component.framework.Constants.Component.JBI_XML;
import static org.ow2.petals.component.framework.Constants.Component.META_INF;

/**
 * The common part of the ServiceUnitManager. It handles service units JBI
 * descriptor during their deployment, life-cycle and undeployment.
 * 
 * @author ofabre - EBM Websourcing
 * @author alouis - EBM Websourcing
 * 
 */
public abstract class ServiceUnitManager implements javax.jbi.component.ServiceUnitManager {

    /**
     * The component's Logger
     */
    protected Logger logger;

    /**
     * The reference to the component
     */
    protected final AbstractComponent component;

    /**
     * The service unit data handlers. Each service unit is linked to a
     * datahandler. Key is the service unit name.
     */
    private final Map<String, ServiceUnitDataHandler> serviceUnitDataHandlers;

    /**
     * The WSDL writer
     */
    private WSDL4ComplexWsdlWriter wsdlWriter;

    /**
     * The WSDL reader
     */
    private WSDL4ComplexWsdlReader wsdlReader;

    /**
     * Creates a new instance of {@link ServiceUnitManager}
     * 
     * @param component
     */
    public ServiceUnitManager(final AbstractComponent component) {
        super();
        this.component = component;
        this.logger = component.getLogger();
        this.serviceUnitDataHandlers = new ConcurrentHashMap<String, ServiceUnitDataHandler>();
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.component.ServiceUnitManager#init(java.lang.String,
     * java.lang.String)
     */
    public void init(final String serviceUnitName, final String suRootPath)
            throws DeploymentException {
        this.logger.log(Level.FINE, "-CALL-ServiceUnitManager.init() serviceUnitName:"
                + serviceUnitName + " -  serviceUnitRootPath:" + suRootPath);

        try {
            final WSDL4ComplexWsdlFactory wsdl4ComplexWsdlFactory = WSDL4ComplexWsdlFactory
                    .newInstance();
            this.wsdlReader = wsdl4ComplexWsdlFactory.newWSDLReader();
            this.wsdlWriter = wsdl4ComplexWsdlFactory.newWSDLWriter();
        } catch (final WSDL4ComplexWsdlException e) {
            throw new DeploymentException(e.getClass().getSimpleName() + " : " + e.getMessage());
        }
    }

    /**
     * (non-Javadoc)
     * 
     * @see javax.jbi.component.ServiceUnitManager#deploy(java.lang.String,
     *      java.lang.String)
     */
    public String deploy(final String serviceUnitName, final String suRootPath)
            throws DeploymentException {
        if (this.component.getContext() == null) {
            // the component has not been initialized for the first time...
            throw new DeploymentException(
                    "Can not deploy a service unit on a component which has not been initialized");
        }

        this.logger.log(Level.FINE, "-CALL-ServiceUnitManager.deploy() serviceUnitName:"
                + serviceUnitName + " - suRootPath:" + suRootPath);

        if (this.serviceUnitDataHandlers.containsKey(serviceUnitName)) {
            throw new DeploymentException(
                    "A service unit with the given name is already registered : " + serviceUnitName);
        } else {
            final File jbiXmlFile = new File(suRootPath + File.separator + META_INF
                    + File.separator + JBI_XML);

            Jbi jbiDescriptor;
            try {
                String jbiXmlString = this.processProperties(jbiXmlFile);
                jbiDescriptor = JBIDescriptorBuilder
                        .buildJavaJBIDescriptor(new ByteArrayInputStream(jbiXmlString.getBytes()));
            } catch (final CDKJBIDescriptorException e) {
                throw new DeploymentException(e);
            } catch (PEtALSCDKException e) {
                throw new DeploymentException(e);
            } catch (IOException e) {
                throw new DeploymentException(e);
            }

            ServiceUnitDataHandler handler = new ServiceUnitDataHandler(serviceUnitName,
                    suRootPath, jbiDescriptor);

            // validate the SU extensions if a dedicated validator is available.
            if (this.component.getServiceUnitExtensionsValidator() != null) {
                try {
                    this.validateExtensions(jbiDescriptor);
                } catch (final PEtALSCDKException e) {
                    throw new DeploymentException(
                            "Failed to validate at least one SU extensions for SU "
                                    + serviceUnitName, e);
                }
            }

            try {
                this.processConsumesNodes(handler);
            } catch (final JBIException ex) {
                throw new DeploymentException("Failed to process consumes nodes for SU "
                        + serviceUnitName, ex);
            }

            this.logger.log(Level.FINEST, "Register a data handler for SU : " + serviceUnitName);
            this.serviceUnitDataHandlers.put(serviceUnitName, handler);

            // additional deployment by component implementation
            if (this.component.getServiceUnitListener() != null) {
                try {
                    this.component.getServiceUnitListener().onSUDeployed(serviceUnitName,
                            suRootPath, jbiDescriptor);
                } catch (final Throwable e) {
                    // clean cache if any exception
                    this.logger.log(Level.FINEST, "Unregister a data handler for SU : "
                            + serviceUnitName);
                    this.serviceUnitDataHandlers.remove(serviceUnitName);

                    throw new DeploymentException(
                            "The ServiceUnit listener failed to process for SU " + serviceUnitName,
                            e);
                }
            }
        }

        final String result = ManagementMessageUtil.getComponentTaskResult(this.component
                .getContext().getComponentName(), "deploy",
                ManagementMessageUtil.TASK_RESULT_SUCCESS);
        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.component.ServiceUnitManager#start(java.lang.String)
     */
    public void start(final String serviceUnitName) throws DeploymentException {
        if (this.component.getContext() == null) {
            // the component has not been initialized for the first time...
            throw new DeploymentException(
                    "Can not start a service unit on a component which has not been initialized");
        }

        this.logger.log(Level.FINE, "-CALL-ServiceUnitManager.start() serviceUnitName:"
                + serviceUnitName);

        final ServiceUnitDataHandler handler = this.serviceUnitDataHandlers.get(serviceUnitName);
        try {
            this.processProvidesNodes(handler);
        } catch (final JBIException ex) {
            final String msg = "Failed to start SU " + serviceUnitName;
            this.logger.log(Level.SEVERE, msg, ex);
            throw new DeploymentException(msg, ex);
        }

        // call the Service Unit Listener if any
        if (this.component.getServiceUnitListener() != null) {
            try {
                this.component.getServiceUnitListener().onSUStarted(serviceUnitName);
            } catch (final PEtALSCDKException e) {
                throw new DeploymentException("The ServiceUnit listener failed to process for SU "
                        + serviceUnitName, e);
            }
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.component.ServiceUnitManager#stop(java.lang.String)
     */
    public void stop(final String serviceUnitName) throws DeploymentException {
        if (this.component.getContext() == null) {
            // the component has not been initialized for the first time...
            throw new DeploymentException(
                    "Can not stop a service unit on a component which has not been initialized");
        }

        this.logger.log(Level.FINE, "-CALL-ServiceUnitManager.stop() serviceUnitName:"
                + serviceUnitName);

        if (this.component.getServiceUnitListener() != null) {
            try {
                this.component.getServiceUnitListener().onSUStopped(serviceUnitName);
            } catch (final PEtALSCDKException e) {
                throw new DeploymentException("The ServiceUnit listener failed to process for SU "
                        + serviceUnitName, e);
            }
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.component.ServiceUnitManager#shutDown(java.lang.String)
     */
    public void shutDown(final String serviceUnitName) throws DeploymentException {
        // stop the providing of services for the given SU, as said in the JBI
        // specifications
        if (this.component.getContext() == null) {
            // the component has not been initialized for the first time...
            throw new DeploymentException(
                    "Can not shutdown a service unit on a component which has not been initialized");
        }

        this.logger.log(Level.FINE, "-CALL-ServiceUnitManager.shutdown() serviceUnitName  "
                + serviceUnitName);

        try {
            final ServiceUnitDataHandler handler = this.serviceUnitDataHandlers
                    .get(serviceUnitName);
            final Iterator<ServiceEndpoint> endpoints = handler.getEpServiceDesc().keySet()
                    .iterator();
            while (endpoints.hasNext()) {
                final ServiceEndpoint endpoint = endpoints.next();

                this.component.getContext().deactivateEndpoint(endpoint);

                // The endpoint removing must be done on the Iterator, not on
                // the Map.
                // Otherwise a ConcurrentModificationException will occurs when
                // shutdowning a SA continaing a SU containing more than one
                // service.
                endpoints.remove();
            }

        } catch (final JBIException e) {
            throw new DeploymentException("Deactivate endpoint failed for SU " + serviceUnitName, e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.component.ServiceUnitManager#undeploy(java.lang.String,
     * java.lang.String)
     */
    public String undeploy(final String serviceUnitName, final String suRootPath)
            throws DeploymentException {
        if (this.component.getContext() == null) {
            // the component has not been initialized for the first time...
            throw new DeploymentException(
                    "Can not undeploy a service unit on a component which has not been initialized");
        }

        this.logger.log(Level.FINE, "-CALL-ServiceUnitManager.undeploy() serviceUnitName:"
                + serviceUnitName + " - suRootPath:" + suRootPath);

        final ServiceUnitDataHandler handler = this.serviceUnitDataHandlers.get(serviceUnitName);

        try {
            if (this.component.getServiceUnitListener() != null) {
                this.component.getServiceUnitListener().onSUUndeployed(serviceUnitName, suRootPath,
                        handler.getDescriptor());
            }
        } catch (final PEtALSCDKException e) {
            // instead of throwing an exception, return an error message
            ManagementMessageUtil.getComponentTaskResult(this.component.getContext()
                    .getComponentName(), "undeploy", ManagementMessageUtil.TASK_RESULT_FAILED,
                    ManagementMessageUtil.MESSAGE_TYPE_ERROR, "ERROR",
                    "The ServiceUnit listener failed to process", new String[] { serviceUnitName,
                            suRootPath }, e.getMessage());
            // throw new DeploymentException("The ServiceUnit listener failed to
            // process", e);
        } finally {
            this.logger.log(Level.FINEST, "Unregister a data handler for SU " + serviceUnitName);
            this.serviceUnitDataHandlers.remove(serviceUnitName);
        }

        return ManagementMessageUtil.getComponentTaskResult(this.component.getContext()
                .getComponentName(), "undeploy", ManagementMessageUtil.TASK_RESULT_SUCCESS);
    }

    /**
     * Return all {@link ServiceUnitDataHandler}s as a Map
     * 
     * @return all {@link ServiceUnitDataHandler}s as a Map
     */
    public Map<String, ServiceUnitDataHandler> getServiceUnitDataHandlers() {
        return this.serviceUnitDataHandlers;
    }

    /**
     * Retrieve the Provides node that has activated the given jbi Endpoint
     * 
     * @param serviceEndpoint
     *            the previously activated jbi Endpoint
     * @return the {@link Provides} element
     */
    public Provides getProvidesFromEndpoint(final ServiceEndpoint serviceEndpoint) {
        for (final ServiceUnitDataHandler dataHandler : this.serviceUnitDataHandlers.values()) {
            final Provides provides = dataHandler.getEpJBIDesc().get(serviceEndpoint);
            if (provides != null) {
                return provides;
            }
        }
        return null;
    }

    /**
     * Retrieve the Consumes node matching the given destination information
     * (endpoint name, service name, interface name). Search order is by
     * ascending priority : endpointName, serviceName, interfaceName. If there
     * are many consumes matching information, return the first one found.
     * 
     * @param endpointName
     *            an endpoint name, can be null
     * @param serviceName
     *            a service {@link QName}, can be null
     * @param interfaceName
     *            an interface {@link QName}, can be null
     * @return the {@link Consumes} element matching information, can be null
     */
    public Consumes getConsumesFromDestination(final String endpointName, final QName serviceName,
            final QName interfaceName) {
        Consumes result = null;
        if (endpointName != null) {
            result = this.getConsumesForEndpoint(endpointName);
        }
        if (result == null) {
            if (serviceName != null) {
                result = this.getConsumesForService(serviceName);
            }
            if (result == null) {
                if (interfaceName != null) {
                    result = this.getConsumesForInterface(interfaceName);
                }
            }
        }
        return result;
    }

    /**
     * Retrieve the Service description (as DOM Document) of the given jbi
     * Endpoint
     * 
     * @param serviceEndpoint
     *            the jbi Endpoint
     * @return the service description as Dom Document
     */
    public Document getServiceDescription(final ServiceEndpoint serviceEndpoint) {
        for (final ServiceUnitDataHandler dataHandler : this.serviceUnitDataHandlers.values()) {
            final Document serviceDescription = dataHandler.getEndpointDescription(serviceEndpoint);
            if (serviceDescription != null) {
                return serviceDescription;
            }
        }
        return null;
    }

    /**
     * Get the SU Data Handler matching the given Consumes/Provides service.
     * 
     * @param consumes
     *            The @ code Consumes} instance
     * @return
     */
    public ServiceUnitDataHandler getSUDataHandlerForService(final Object service) {
        ServiceUnitDataHandler suDataHandler = null;

        if (service instanceof Consumes) {
            suDataHandler = this.getSUDataHandlerForConsumes((Consumes) service);
        } else if (service instanceof Provides) {
            suDataHandler = this.getSUDataHandlerForProvides((Provides) service);
        }

        return suDataHandler;
    }

    /**
     * Get the SU Data Handler matching the given Provides.
     * 
     * @param consumes
     *            The @ code Consumes} instance
     * @return
     */
    public ServiceUnitDataHandler getSUDataHandlerForConsumes(final Consumes consumes) {
        for (final ServiceUnitDataHandler dataHandler : this.serviceUnitDataHandlers.values()) {
            for (final Consumes consumesElement : dataHandler.getDescriptor().getServices()
                    .getConsumes()) {
                if (consumes.equals(consumesElement)) {
                    return dataHandler;
                }
            }
        }
        return null;
    }

    /**
     * Get the SU Data Handler matching the given Consumes.
     * 
     * @param provides
     *            The @ code Provides} instance
     * @return
     */
    public ServiceUnitDataHandler getSUDataHandlerForProvides(final Provides provides) {
        for (final ServiceUnitDataHandler dataHandler : this.serviceUnitDataHandlers.values()) {
            for (final Provides providesElement : dataHandler.getDescriptor().getServices()
                    .getProvides()) {
                if (provides.equals(providesElement)) {
                    return dataHandler;
                }
            }
        }
        return null;
    }

    /**
     * Search the wsdl description of the activated endpoint in the Provides
     * extensions. If no wsdl description can be found, it creates a light
     * weight Wsdl description from Provides Interface, Service and Endpoint
     * names.
     * 
     * @param installRoot
     *            the SU install root
     * @param provides
     *            the Provices node
     * @param autogeneratedEndpointName
     *            the autogenerated Endpoint Name. Can be {@code null}
     * @return the WSDL description as Dom Document
     */
    protected Document getEnpointWSDLDesc(final String installRoot, final Provides provides,
            String autogeneratedEndpointName) throws PEtALSCDKException {
        Document wsdlDesc = null;

        String wsdlFileLocation = provides.getWsdl();
        final Settableboolean wsdlImportsDownload = provides.getWsdlImportsDownload();
        final boolean isWsdlImportsDownload = wsdlImportsDownload == null ? true
                : wsdlImportsDownload.isValue();
        if (!StringHelper.isNullOrEmpty(wsdlFileLocation)) {
            URL wsdlLocation = null;
            try {
                wsdlLocation = new URL(wsdlFileLocation);
            } catch (final MalformedURLException e) {
                // Can be a local file
                if (!wsdlFileLocation.startsWith(File.separator)) {
                    wsdlFileLocation = File.separator + wsdlFileLocation;
                }

                final File wsdlFile = new File(installRoot + wsdlFileLocation);
                if (!wsdlFile.exists()) {
                    throw new PEtALSCDKException("Can not retrieve WSDL document from location : "
                            + wsdlFileLocation);
                }
                try {
                    wsdlLocation = wsdlFile.toURL();
                } catch (final MalformedURLException e1) {
                    // not possible
                }
            }
            try {
                final Description wsdlDescription = this.wsdlReader
                        .readWSDL4ComplexWSDL(wsdlLocation.toURI());
                if (isWsdlImportsDownload) {
                    wsdlDescription.addImportedDocumentsInWsdl();
                }
                if (autogeneratedEndpointName != null) {
                    // change arbitrary all the endpoint named
                    // 'petals:autogenerated' found to the given auto-generated
                    // value
                    for (final Service service : wsdlDescription.getServices()) {
                        final Endpoint autogeneratedEndpoint = service
                                .getEndpoint(AUTOGENERATED_ENDPOINT_NAME);
                        if (autogeneratedEndpoint != null) {
                            autogeneratedEndpoint.setName(autogeneratedEndpointName);
                        }
                    }
                    final String wsdlString = this.wsdlWriter
                            .writeWSDL4ComplexWsdl(wsdlDescription);

                    try {
                        final OutputStream outputStream = new FileOutputStream(new File(
                                wsdlLocation.getFile()));
                        outputStream.write(wsdlString.getBytes());
                        outputStream.close();
                    } catch (final IOException e) {
                        throw new PEtALSCDKException(
                                "Failed to update the auto-generated endpoint name in the WSDL. Caused:"
                                        + e.getClass().getSimpleName() + " : " + e.getMessage());
                    }
                }
                wsdlDesc = this.wsdlWriter.getDocument(wsdlDescription);
            } catch (final WSDL4ComplexWsdlException e) {
                throw new PEtALSCDKException("Failed to parse the WSDL. Caused:"
                        + e.getClass().getSimpleName() + " : " + e.getMessage());
            } catch (final URISyntaxException e) {
                // not possible
            }

        } else {
            // if the WSDL field is null, generate a basic WSDL
            try {
                wsdlDesc = WSDLHelper.getInstance().createLightWSDL20(provides.getInterfaceName(),
                        provides.getServiceName(), provides.getEndpointName());
            } catch (final WSDLException e) {
                throw new PEtALSCDKException("Can not auto-generate a WSDL document", e);
            }
        }

        return wsdlDesc;
    }

    /**
     * Process the interceptors for a consumes/provides service.
     * 
     * @param handler
     * @param service
     *            the {@code Provides} or {@code Consumes} service
     * @throws JBIException
     */
    protected void processInterceptors(final ServiceUnitDataHandler handler, final Object service)
            throws JBIException {

        SUInterceptors suInterceptors = null;

        if (service instanceof Consumes) {
            suInterceptors = ((Consumes) service).getSuInterceptors();
        } else if (service instanceof Provides) {
            suInterceptors = ((Provides) service).getSuInterceptors();
        }

        // create the manager for this extension element. Each element has a
        // linked manager with preloaded interceptors.

        final MessageInterceptorManager interceptorManager;
        if (this.component.getComponentConfiguration().getPropertiesFile() != null
                && !StringHelper.isNullOrEmpty(this.component.getComponentConfiguration()
                        .getPropertiesFile().getValue())) {
            interceptorManager = new MessageInterceptorManager(this.component
                    .getComponentConfiguration().getPropertiesFile().getValue(), this.logger);
        } else {
            interceptorManager = new MessageInterceptorManager(this.logger);
        }

        List<SUInterceptor> sendSUInterceptors = null;
        List<SUInterceptor> sendResponseSUInterceptors = null;
        List<SUInterceptor> acceptSUInterceptors = null;
        List<SUInterceptor> acceptResponseSUInterceptors = null;
        if (suInterceptors != null) {
            if (suInterceptors.getSend() != null) {
                sendSUInterceptors = suInterceptors.getSend().getInterceptor();
            }
            if (suInterceptors.getSendResponse() != null) {
                sendResponseSUInterceptors = suInterceptors.getSendResponse().getInterceptor();
            }
            if (suInterceptors.getAccept() != null) {
                acceptSUInterceptors = suInterceptors.getAccept().getInterceptor();
            }
            if (suInterceptors.getAcceptResponse() != null) {
                acceptResponseSUInterceptors = suInterceptors.getAcceptResponse().getInterceptor();
            }
        }

        interceptorManager.init(this.component.getInterceptors(), acceptSUInterceptors,
                sendSUInterceptors, acceptResponseSUInterceptors, sendResponseSUInterceptors);

        handler.addMessageInterceptorManager(service, interceptorManager);
    }

    /**
     * 
     * @param handler
     * @throws JBIException
     */
    protected void processConsumesNodes(final ServiceUnitDataHandler handler) throws JBIException {
        final Jbi descriptor = handler.getDescriptor();
        final List<Consumes> consumesList = descriptor.getServices().getConsumes();
        for (final Consumes consumes : consumesList) {
            this.processInterceptors(handler, consumes);
        }
    }

    /**
     * 
     * @param handler
     * @throws JBIException
     */
    protected void processProvidesNodes(final ServiceUnitDataHandler handler) throws JBIException {
        final Jbi descriptor = handler.getDescriptor();

        final List<Provides> providesList = descriptor.getServices().getProvides();
        for (final Provides provides : providesList) {
            final ServiceEndpoint sep = this.processProvidesNode(handler, provides);
            this.logger.log(Level.INFO, "New Service Endpoint has been deployed : " + sep);
        }
    }

    /**
     * Process the given provides node.
     * 
     * @param handler
     * @param provides
     * @return The resulted Service Endpoint
     * @throws JBIException
     */
    protected ServiceEndpoint processProvidesNode(final ServiceUnitDataHandler handler,
            final Provides provides) throws JBIException {

        String endpointName = provides.getEndpointName();
        String autogeneratedEndpointName = null;

        if (AUTOGENERATED_ENDPOINT_NAME.equals(endpointName)) {
            endpointName = IDGenerator.getInstance().getNewID();
            autogeneratedEndpointName = endpointName;

            // update the provides object to set the real endpoint
            provides.setEndpointName(endpointName);

            final File jbiXmlFile = new File(handler.getInstallRoot() + File.separator + META_INF
                    + File.separator + JBI_XML);

            // update the descriptor of the SU to set the real endpoint
            // then, if the SU is restarted, the real endpoint is
            // used
            try {
                JBIDescriptorBuilder.buildXmlJBIdescriptor(handler.getDescriptor(),
                        new FileOutputStream(jbiXmlFile));
            } catch (final FileNotFoundException e) {
                throw new JBIException("Failed to persist the auto-generated endpoint name", e);
            } catch (final CDKJBIDescriptorException e) {
                throw new JBIException("Failed to persist the auto-generated endpoint name", e);
            }
        }

        final Document wsdlDescription = this.getEnpointWSDLDesc(handler.getInstallRoot(),
                provides, autogeneratedEndpointName);
        this.processInterceptors(handler, provides);

        final ServiceEndpoint ep = this.component.getContext().activateEndpoint(
                provides.getServiceName(), endpointName);
        handler.addEndpointDescription(ep, wsdlDescription);
        handler.addJBIDescription(ep, provides);

        return ep;
    }

    /**
     * Validate the SU extensions.
     * 
     * @param descriptor
     *            the service unit descriptor
     * @throws PEtALSCDKException
     *             if at least an extension block is not valid. It is the role
     *             of the developer to specify which extension is not valid.
     */
    protected void validateExtensions(final Jbi descriptor) throws PEtALSCDKException {

        // validate the provides
        // TODO : Deprecated when the component specific XSD validation would be
        // available
        final List<Provides> providesList = descriptor.getServices().getProvides();
        for (final Provides provides : providesList) {
            this.component.getServiceUnitExtensionsValidator().validateProvidesExtensions(
                    new ConfigurationExtensions(provides.getAny()));
        }

        // validate the consumes
        // TODO : Deprecated when the component specific XSD validation would be
        // available
        final List<Consumes> consumesList = descriptor.getServices().getConsumes();
        for (final Consumes consumes : consumesList) {
            this.component.getServiceUnitExtensionsValidator().validateConsumesExtensions(
                    new ConfigurationExtensions(consumes.getAny()));
        }
    }

    /**
     * Retrieve the install root of the Service Unit that has activated the
     * given endpoint
     * 
     * @param endpoint
     *            The activated endpoint
     * @return the Service unit install root
     */
    protected String getSUInstallRootForActivatedEp(final ServiceEndpoint endpoint) {
        return this.getSUDataHandlerForEndpoint(endpoint).getInstallRoot();
    }

    /**
     * Retrieve the data handler of the Service Unit that has activated the
     * given endpoint.
     * 
     * @param endpoint
     *            the
     * @return the {@link ServiceUnitDataHandler}
     */
    public ServiceUnitDataHandler getSUDataHandlerForEndpoint(final ServiceEndpoint endpoint) {
        for (final ServiceUnitDataHandler dataHandler : this.serviceUnitDataHandlers.values()) {
            for (final ServiceEndpoint endpoint2 : dataHandler.getEpJBIDesc().keySet()) {
                if (endpoint.equals(endpoint2)) {
                    return dataHandler;
                }
            }
        }
        return null;
    }

    /**
     * Get the consumes matching the given endpoint name.
     * 
     * @param serviceName
     *            The endpoint name to match
     * @return The {@code Consumes} instance if matched, @ null} otherwise
     */
    public Consumes getConsumesForEndpoint(final String endpointName) {
        for (final ServiceUnitDataHandler dataHandler : this.serviceUnitDataHandlers.values()) {
            for (final Consumes consumes : dataHandler.getDescriptor().getServices().getConsumes()) {
                if (endpointName.equals(consumes.getEndpointName())) {
                    return consumes;
                }
            }
        }
        return null;
    }

    /**
     * Get the consumes matching the given service name.
     * 
     * @param serviceName
     *            The service qualified name to match
     * @return The {@code Consumes} instance if matched, @ null} otherwise
     */
    public Consumes getConsumesForService(final QName serviceName) {
        for (final ServiceUnitDataHandler dataHandler : this.serviceUnitDataHandlers.values()) {
            for (final Consumes consumes : dataHandler.getDescriptor().getServices().getConsumes()) {
                if (serviceName.equals(consumes.getServiceName())) {
                    return consumes;
                }
            }
        }
        return null;
    }

    /**
     * Get the consumes matching the given service name.
     * 
     * @param serviceName
     *            The service qualified name to match
     * @return The {@code Consumes} instance if matched, @ null} otherwise
     */
    public Consumes getConsumesForInterface(final QName interfaceName) {
        for (final ServiceUnitDataHandler dataHandler : this.serviceUnitDataHandlers.values()) {
            for (final Consumes consumes : dataHandler.getDescriptor().getServices().getConsumes()) {
                if (interfaceName.equals(consumes.getInterfaceName())) {
                    return consumes;
                }
            }
        }
        return null;
    }

    /**
     * Get the {@code set} of endpoints exposed by the given Service Unit.
     * 
     * @param serviceUnitName
     *            The name of the Service Unit
     * @return a {@code set} of {@code ServiceEndpoint}
     */
    public Set<ServiceEndpoint> getEndpointsForServiceUnit(final String serviceUnitName) {
        return this.serviceUnitDataHandlers.get(serviceUnitName).getEpServiceDesc().keySet();
    }

    /**
     * Replace the component properties by their values in the JBI descriptor,
     * if the component properties file is defined.
     * 
     * @param jbiFile
     * @return the JBI descriptor in String form
     * @throws IOException
     * @throws PEtALSCDKException
     */
    private String processProperties(File jbiFile) throws IOException, PEtALSCDKException {
        FileInputStream jbiFileInputStream = new FileInputStream(jbiFile);
        String jbiString = org.ow2.petals.component.framework.util.SourceHelper
                .convertToString(new StreamSource(jbiFileInputStream));
        jbiFileInputStream.close();

        if (this.component.getComponentConfiguration().getPropertiesFile() != null
                && !StringHelper.isNullOrEmpty(this.component.getComponentConfiguration()
                        .getPropertiesFile().getValue())) {

            Properties componentProperties = PropertiesUtil.loadProperties(this.component
                    .getComponentConfiguration().getPropertiesFile().getValue());
            for (Object key : componentProperties.keySet()) {
                jbiString = jbiString.replaceAll("\\$\\{" + key + "\\}",
                        (String) componentProperties.get(key));
            }
        }

        return jbiString;
    }
}
